#include <formatio.h>
#include <rs232.h>
#include <utility.h>
#include <ansi_c.h>
#include <userint.h>
#include "Communication.h"

#define RUN_MODE TRUE  // TRUE, FALSE

// private variables
char 	fRS232_Command[8];           		// RS232 command send block 
char 	fRS232_Package[8];           		// RS232 data receive block 
char 	fRS232_XCommand[10];         		// extended RS232 command send block 
char 	fRS232_XPackage[10];         		// extended RS232 data receive block 


int RS232initialise(int aPort, int aBaud, double aTimeOut)
{
   int result;
  
   if (!RUN_MODE) return TRUE;		// set for testing  
   result = OpenComConfig(aPort, "",aBaud,NO_PARITY,BITS_8,STOPBIT_1,IN_Q_512,OUT_Q_512);
   if (result >= 0) SetComTime(aPort,aTimeOut);

   return result;
}

void RS232release(int aPort)
{
   if (RUN_MODE) CloseCom(aPort);
}

int RS232try(int aPort, double aTimeOut)
{
   int result;
  
   SetComTime(aPort, 1);       
   result = ComRdByte(aPort);
   SetComTime(aPort,aTimeOut);
  
   return result;
}

//Synchronises RS232 communication. RETURN: TRUE - sync vSuccessful, FALSE - sync failure
//In previous version used for Attenuator calibration, the sync char was chosen to be 0x20. The new sync char is now 0xFF
int RS232synchronise(int aPort, double aTimeOut)
{
  int i;
  char c[2];

  c[1] = (~0xFD) & 0xFF;
  SetComTime(aPort, 0.1);       /* set time out to 0.1 sec */

  for (i = 0; i < MAX_SYNC_TRY; i++) {
    ComWrtByte(aPort,0xFF);
    c[0] = ComRdByte(aPort) & 0xFF;
    if (c[0] == c[1]) {
      SetComTime(aPort,aTimeOut);      /* set time out to default again */
      return(TRUE);
      }
    ProcessSystemEvents();  
    }
  SetComTime(aPort,aTimeOut);      /* set time out to default again */

  return FALSE;
}

int RS232writePackage(int aPort, double aTimeOut)
{
  int n, c;
  
  if (!RUN_MODE) return TRUE;                // set for testing
  
  c = (~(fRS232_Command[0] + fRS232_Command[1] + fRS232_Command[2])) & 0x00FF; // calculate inverted checksum 
  fRS232_Command[3] = (char) c;

  FlushOutQ(aPort); if (ComWrt(aPort,fRS232_Command,COMMAND_LEN) != COMMAND_LEN) return FALSE;
  FlushInQ(aPort); n = ComRdByte(aPort);
  if (n < 0) return FALSE;
  else n = n & 0x00FF;        // read the reply checksum 

  if (n != c) {                         // checksum error, maybe out of sync 
    if (RS232synchronise(aPort,aTimeOut) != TRUE) return FALSE;  // can not sync, real trouble 
    n = -1;                             // n = -1 so we can re-try 
    }

  if (n == -1) {                        // Comm port time out, try another time 
    FlushOutQ(aPort); 
    if (ComWrt(aPort,fRS232_Command,COMMAND_LEN) != COMMAND_LEN) return FALSE;
    
    FlushInQ(aPort);
    n = ComRdByte(aPort) & 0x00FF;      // read the reply checksum 
    }

  if (n != c) return FALSE;
  else return TRUE;
}

int RS232writeNoSync(int aPort, double aTimeOut)
{
  int n, c;
  
  if (!RUN_MODE) return TRUE;                // set for testing
  
  c = (~(fRS232_Command[0] + fRS232_Command[1] + fRS232_Command[2])) & 0x00FF; // calculate inverted checksum 
  fRS232_Command[3] = (char) c;

  FlushOutQ(aPort); if (ComWrt(aPort,fRS232_Command,COMMAND_LEN) != COMMAND_LEN) return FALSE;
  FlushInQ(aPort);

  SetComTime(aPort, 1);       /* set time out to 0.1 sec */
  n = ComRdByte(aPort);
  SetComTime(aPort,aTimeOut);   /* set time out to default again */

  if (n < 0) return FALSE;
  else n = n & 0x00FF;        // read the reply checksum 

//if (n != c) {                         // checksum error, maybe out of sync 
//  if (RS232synchronise(aPort,aTimeOut) != TRUE) return FALSE;  // can not sync, real trouble 
    n = -1;                             // n = -1 so we can re-try 
// }

  if (n == -1) {                        // Comm port time out, try another time 
    FlushOutQ(aPort); 
    if (ComWrt(aPort,fRS232_Command,COMMAND_LEN) != COMMAND_LEN) return FALSE;
    
    FlushInQ(aPort);
    n = ComRdByte(aPort) & 0x00FF;      // read the reply checksum 
    }

  if (n != c) return FALSE;
  else return TRUE;
}

//The data read are stored in the global array named fRS232_Package[8];
int RS232readPackage(int aPort)
{
  int i, n, c;

  if (!RUN_MODE) return TRUE;		// set for testing
  for (i = 0; i < 4; i++) fRS232_Package[i] = 0xFF;		  /* initialise array */

  for (i = 0; i < 10; i++) if (GetInQLen(aPort) >= 4) break;		//wait for package receives
    
  n = ComRd(aPort,fRS232_Package,4); 

  if (n != 4) return RS232READPACKAGE_ERROR_RETURNNUMOFBYTES;
  else {
    c = ~(fRS232_Package[0] + fRS232_Package[1] + fRS232_Package[2]);
    n = fRS232_Package[3];
    if ((c & 0x00FF) != (n & 0x00FF)) return RS232READPACKAGE_ERROR_RETURNCHECKSUM;
	  return TRUE;
    }
}

/*****************************************************************
utiities
******************************************************************/

//lambda (nm) = 600 + 5 * lambda-code
int LambdaToLambdaCode(int aLambda)
{
  return (int)((aLambda-600)/5);
}

//Calculates lambda-code from lambda: lambda = 600 + 5 * lambda-code
int LambdaCodeToLambda(unsigned char aLambdaCode)
{
  return (int)(600 + 5 * aLambdaCode);
}

//Returns the lower byte of an two-byte integer
char LoByte(int aInt)
{
  return aInt & 0x00FF;
}

//Returns the higher byte of an two-byte integer
char HiByte(int aInt)
{
  return (aInt>>8) & 0x00FF;
}

//Returns a 16bit integer made up of two bytes
int MakeWord(int aHiByte, int aLoByte)
{
  aHiByte &= 0xFF;
  aLoByte &= 0xFF;

  return ((aHiByte << 8) | aLoByte);
}


/***************************************************************************
Attenuator RPC commands
***************************************************************************/

int AttenuatorGroup_entry(int aPort, double aTimeOut)
{
  fRS232_Command[0] = 0x70;
  fRS232_Command[1] = 0x43;
  fRS232_Command[2] = 0x5B;
  return RS232writeNoSync(aPort, aTimeOut);
}

int AttenuatorGroup_exit(int aPort, double aTimeOut)
{
  fRS232_Command[0] = 0x9B;
  fRS232_Command[1] = 0x43;
  fRS232_Command[2] = 0x5B;
  return RS232writePackage(aPort, aTimeOut);
}

int AttenuatorGroup_selectLambda(int aLambda, int aPort, double aTimeOut)
{
  fRS232_Command[0] = 0x88;
  fRS232_Command[1] = (char) aLambda;
  fRS232_Command[2] = 0x5B;
  return RS232writePackage(aPort, aTimeOut);
}

int AttenuatorGroup_setLambda(int aPort, double aTimeOut)
{
  fRS232_Command[0] = 0x89;
  fRS232_Command[1] = 0x43;
  fRS232_Command[2] = 0x5B;
  return RS232writePackage(aPort, aTimeOut);
}

int AttenuatorGroup_setAttenuation(double aAttenuation, int aPort, double aTimeOut)
{
  int n;
  char buf[260];
  
  if (aAttenuation > 60) aAttenuation = 60.0;
  aAttenuation = aAttenuation/0.05;
  Fmt(buf,"%s<%f[p0]",aAttenuation); Scan(buf,"%s>%d",&n);

  fRS232_Command[0] = 0x8B;
  fRS232_Command[1] = HiByte(n);
  fRS232_Command[2] = LoByte(n);
  return RS232writePackage(aPort, aTimeOut);
}

